@@ -341,13 +341,6 @@ class Baseplate:
                 )
             )
 
-            # If statsd metrics are currently configured, then enable Prometheus metrics simultaneous.
-            # Once the migration to Prometheus is complete, then only enable Prometheus metrics.
-            from baseplate.observers.prometheus import PrometheusBaseplateObserver
-
-            observer = PrometheusBaseplateObserver()
-            self.register(observer)
-
         elif "metrics.namespace" in self._app_config:
             from baseplate.lib.metrics import metrics_client_from_config
             from baseplate.observers.metrics import MetricsBaseplateObserver
@@ -359,13 +352,6 @@ class Baseplate:
                 )
             )
 
-            # If statsd metrics are currently configured, then enable Prometheus metrics simultaneous.
-            # Once the migration to Prometheus is complete, then only enable Prometheus metrics.
-            from baseplate.observers.prometheus import PrometheusBaseplateObserver
-
-            observer = PrometheusBaseplateObserver()
-            self.register(observer)
-
         else:
             skipped.append("metrics")
 
